﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// Tämän rivin lisäämällä saamme käyttöömme NavMeshAgent-luokan
using UnityEngine.AI;

public class VihollisenLiikkuminen : MonoBehaviour {

    // NavMeshAgent komponenttia käytetään vihollisen liikuttamiseen.
    NavMeshAgent agentti;

    // Tallennetaan tähän viittaus pelaajan robotti-hahmoon, jotta tiedämme minne liikuttaa vihollista.
    Transform robotti;

    // Käytetään tätä muuttujaa, kun katsomme onko pelaaja liikkunut viholliselle viimeksi asettamastamme määränpäästä.
    Vector3 määränpää;
    
    // Tallennetaan tähän viittaus robotin kuolemisskriptiin, jotta voimme vähentää sen elämäpisteitä.
    RobotinKuoleminen robotinKuoleminen;

    // Robotille aiheuttaman vaurion määrä.
    public float hyökkäysLujuus = 15f;

    // Millä nopeudella vihollinen voi iskeä. Mitä suurempi arvo, sitä hitaammalla taajudella se hyökkää.
    public float hyökkäysNopeus = 2f;

    // Varastoidaan tähän muuttujaan viime hyökkäyksen ajanhetki, jota käytämme kun haluamme tietää voiko vihollinen hyökätä.
    float viimeHyökkäys;

	void Start () {
        // Tallennetaan viittaus NavMeshAgent-komponenttiin
        agentti = GetComponent<NavMeshAgent>();

        // Start-funktiossa agentilla ei vielä ole määränpäätä, jolloin agentti.destination palauttaa vihollisen sijainnin.
        määränpää = agentti.destination;

        // Etsitään maailmasta 'Robotti'-niminen objekti GameObject.Find-funktiota käyttämällä.
        // Find-funktio palauttaa GameObject-tyyppisen muuttujan. Otetaan tältä sen Transform-komponentti.
        robotti = GameObject.Find("Robotti").transform;

        // Tallennetaan viittaus robotille asetettuun RobotinKuoleminen-komponenttiin
        robotinKuoleminen = robotti.GetComponent<RobotinKuoleminen>();
	}
	
	void Update () {
        // Viime hyökkäyksestä on täytynyt kulua vähintään hyökkäysnopeus-muuttujan verran aikaa.
        // Tämä lasketaan vähentämällä nykyisestä Time.time arvosta viimeHyökkäys-muuttujan arvo
        // ja verrataan sitä hyökkäysnopeus-muuttujan arvoon.
        if (Time.time - viimeHyökkäys >= hyökkäysNopeus)
        {
            // Lisäksi vihollisen on oltava robotin lähellä. Tämä ehto toteutuu, jos vihollisen NavMeshAgentilla on reitti,
            // eli se on matkalla robotin luo. Agentin reitillä jäljellä oleva etäisyys määränpäästä on oltava pienempi tai
            // yhtäsuuri kuin sen stoppingDistance, eli etäisyys määränpäästä, jolloin se pysähtyy.
            if (agentti.hasPath && agentti.remainingDistance <= agentti.stoppingDistance)
            {
                // Vähennetään robotin elämäpisteitä vihollisen hyökkäyslujuus-muuttujan verran.
                robotinKuoleminen.Elämäpisteet -= hyökkäysLujuus;
                
                // Päivitetään elämäpistepalkin täyttyvyys robotin uudella elämäpistemäärällä.
                // Pääsemme Päivitä-funktioon käsiksi helposti staattisen ElämäpisteetPalkki-luokan instanssin kautta.
                ElämäpisteetPalkki.instanssi.Päivitä(robotinKuoleminen.Elämäpisteet);

                // Lopuksi tallennetaan ajanhetki, jolloin hyökkäys tapahtui saadaksemme tietää milloin
                // vihollinen voi taas hyökätä.
                viimeHyökkäys = Time.time;
            }
        }

        // Koska NavMeshAgentille uuden määränpään asettaminen laittaa pelimoottorin laskemaan uuden reitin,
        // on järkevää suoritustehon kannalta asettaa uusi määränpää vain, jos sille on tarvetta.
        // Tarkkaillaan jatkuvasti pelaajan liikettä Update-funktiossa. Tiedämme pelaajan liikkuneen,
        // jos sen sijainnin ja vihollisen nykyisen määränpään etäisyys on suurempi kuin esimerkiksi yksi.

        // Vector3.Distance-funktio palauttaa kahden maailman pisteen etäisyyden toisistaan.
        // Syötetään parametreiksi robotin sijanti ja nykyinen määränpää-muuttujan arvo.
        // Laitetaan tämä if-ehtolauseen sisään ja laitetaan ehtolause toteutumaan, jos etäisyys on suurempi kuin yksi.
        if (Vector3.Distance(robotti.position, määränpää) > 1.0f)
        {
            // Asetetaan määränpää-muuttujalle robotin sijainti.
            määränpää = robotti.position;

            // Laitetaan vihollisen NavMeshAgent-komponentin määränpääksi määränpää-muuttujan arvo.
            // Nyt pelimootori laskee uuden reitin pelaajan luokse ja kulkee sitä pitkin.
            agentti.destination = määränpää;
        }
	}
}
